'use strict';

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var p = require('path');
var babel = require('babel-core');
var template = require('babel-template');
// const printAST = require('ast-pretty-print')
var requireFromString = require('require-from-string');

module.exports = {
  replace,
  looksLike,
  isCodegenComment,
  getReplacement,
  stringToAST,
  resolveModuleToString,
  isPropertyCall
};

function getReplacement(_ref) {
  var stringToCodegen = _ref.string,
      filename = _ref.filename;

  var _babel$transform = babel.transform(stringToCodegen, {
    filename
  }),
      transformed = _babel$transform.code;

  var val = requireFromString(transformed, filename);
  return stringToAST(val);
}

function stringToAST(string) {
  if (typeof string !== 'string') {
    throw new Error('codegen: Must module.exports a string.');
  }
  return template(string, {
    sourceType: 'module',
    preserveComments: true,
    plugins: [
    // add more on request...
    'jsx']
  })();
}

function replace(_ref2) {
  var path = _ref2.path,
      string = _ref2.string,
      filename = _ref2.filename;

  var replacement = getReplacement({
    string,
    filename
  });
  if (!replacement) {
    path.remove();
  } else if (Array.isArray(replacement)) {
    path.replaceWithMultiple(replacement);
  } else {
    path.replaceWith(replacement);
  }
}

function resolveModuleToString(_ref3) {
  var args = _ref3.args,
      filename = _ref3.filename,
      source = _ref3.source;

  var argValues = args.map(function (a) {
    var result = a.evaluate();
    if (!result.confident) {
      throw new Error('codegen cannot determine the value of an argument in codegen.require');
    }
    return result.value;
  });
  var absolutePath = p.join(p.dirname(filename), source.node.value);
  try {
    // allow for transpilation of required modules
    require('babel-register');
  } catch (e) {
    // ignore error
  }
  var mod = require(absolutePath);
  mod = mod && mod.__esModule ? mod.default : mod;
  if (argValues.length) {
    if (typeof mod !== 'function') {
      throw new Error(`\`codegen.require\`-ed module (${source.node.value}) cannot accept arguments because it does not export a function. You passed the arguments: ${argValues.join(', ')}`);
    }
    mod = mod.apply(undefined, _toConsumableArray(argValues));
  }
  return mod;
}

function isCodegenComment(comment) {
  var normalisedComment = comment.value.trim().split(' ')[0].trim();
  return normalisedComment.startsWith('codegen') || normalisedComment.startsWith('@codegen');
}

function isPropertyCall(path, name) {
  return looksLike(path, {
    node: {
      type: 'CallExpression',
      callee: {
        property: { name }
      }
    }
  });
}

function looksLike(a, b) {
  return a && b && Object.keys(b).every(function (bKey) {
    var bVal = b[bKey];
    var aVal = a[bKey];
    if (typeof bVal === 'function') {
      return bVal(aVal);
    }
    return isPrimitive(bVal) ? bVal === aVal : looksLike(aVal, bVal);
  });
}

function isPrimitive(val) {
  // eslint-disable-next-line
  return val == null || /^[sbn]/.test(typeof val);
}

/*
eslint
  complexity: ["error", 8],
  import/no-unassigned-import: "off",
  import/no-dynamic-require: "off",
*/