'use strict';

var _require = require('./replace'),
    asProgram = _require.asProgram,
    asIdentifier = _require.asIdentifier,
    asImportDeclaration = _require.asImportDeclaration;

var _require2 = require('./helpers'),
    isCodegenComment = _require2.isCodegenComment,
    looksLike = _require2.looksLike;

module.exports = codegenPlugin;

function codegenPlugin(_ref) {
  var transformFromAst = _ref.transformFromAst;

  return {
    name: 'codegen',
    visitor: {
      Program(path, _ref2) {
        var filename = _ref2.file.opts.filename;

        var firstNode = path.node.body[0] || {};
        var comments = firstNode.leadingComments || [];
        var isCodegen = comments.some(isCodegenComment);

        if (isCodegen) {
          comments.find(isCodegenComment).value = ' this file was codegened';
          asProgram(transformFromAst, path, filename);
        }
      },
      Identifier(path, _ref3) {
        var filename = _ref3.file.opts.filename;

        var isCodegen = path.node.name === 'codegen';
        if (isCodegen) {
          asIdentifier(path, filename);
        }
      },
      ImportDeclaration(path, _ref4) {
        var filename = _ref4.file.opts.filename;

        var isCodegen = looksLike(path, {
          node: {
            source: {
              leadingComments(comments) {
                return comments && comments.some(isCodegenComment);
              }
            }
          }
        });
        if (isCodegen) {
          asImportDeclaration(path, filename);
        }
      }
    }
  };
}